@@ -6,7 +6,7 @@
 # expressions, contextual output, highlighting, detection and exclusion of
 # nontext files, and complex matching criteria.
 
-# $Id: glark.rb,v 1.1 2006/04/12 11:47:42 jeugenepace Exp $
+# $Id: glark.rb,v 1.2 2006/08/29 22:28:33 jeugenepace Exp $
 
 require "English"
 require 'singleton'
@@ -15,7 +15,7 @@ $stdout.sync = true             # unbuffer
 $stderr.sync = true             # unbuffer
 
 $PACKAGE = "glark"
-$VERSION = "1.7.9"
+$VERSION = "1.7.10"
 
 
 module Text
@@ -980,11 +980,25 @@ class Glark
     search_file(bf)
   end
 
+  def basename_skip?(fname)
+    basename = File.basename(fname)
+    options = GlarkOptions.instance
+    inc = options.with_basename    && !options.with_basename.match(basename)
+    exc = options.without_basename &&  options.without_basename.match(basename)
+    inc || exc
+  end
+
+  def fullname_skip?(fname)
+    options = GlarkOptions.instance
+    inc = options.with_fullname    && !options.with_fullname.match(fname)
+    exc = options.without_fullname &&  options.without_fullname.match(fname)
+    inc || exc
+  end
+
   def skipped?(fname)
     options = GlarkOptions.instance
-    ((options.basename && !options.basename.match(File.basename(fname))) ||
-     (options.fullname && !options.fullname.match(fname)) ||
-     (options.size_limit && File.size(fname) > options.size_limit))
+    basename_skip?(fname) || fullname_skip?(fname) ||
+     (options.size_limit && File.size(fname) > options.size_limit)
   end
 
   def search_text(fname)
@@ -1915,8 +1929,14 @@ class GlarkHelp
     puts "  -0[nnn]                        Use \\nnn as the input record separator"
     puts "  -d, --directories=ACTION       Process directories as read, skip, or recurse"
     puts "      --binary-files=TYPE        Treat binary files as TYPE"
-    puts "      --basename, --name EXPR    Search only files with base names matching EXPR"
-    puts "      --fullname, --path EXPR    Search only files with full names matching EXPR"
+    puts "      --[with-]basename, "
+    puts "      --[with-]name EXPR         Search only files with base names matching EXPR"
+    puts "      --without-basename, "
+    puts "      --without-name EXPR        Ignore files with base names matching EXPR"
+    puts "      --[with-]fullname, "
+    puts "      --[with-]path EXPR         Search only files with full names matching EXPR"
+    puts "      --without-fullname, "
+    puts "      --without-path EXPR        Ignore files with full names matching EXPR"
     puts "  -M, --exclude-matching         Ignore files with names matching the expression"
     puts "  -r, --recurse                  Recurse through directories"
     puts "      --size-limit=SIZE          Search only files no larger than SIZE"
@@ -1987,7 +2007,6 @@ class GlarkOptions
   include Loggable, Singleton
 
   attr_accessor :after
-  attr_accessor :basename
   attr_accessor :before
   attr_accessor :binary_files
   attr_accessor :count
@@ -2000,7 +2019,6 @@ class GlarkOptions
   attr_accessor :file_highlight
   attr_accessor :file_names_only
   attr_accessor :filter
-  attr_accessor :fullname
   attr_reader   :highlight
   attr_accessor :highlighter
   attr_accessor :invert_match
@@ -2023,6 +2041,10 @@ class GlarkOptions
   attr_accessor :version
   attr_accessor :whole_lines
   attr_accessor :whole_words
+  attr_accessor :with_basename
+  attr_accessor :without_basename
+  attr_accessor :with_fullname
+  attr_accessor :without_fullname
   attr_accessor :write_null
   attr_accessor :out
   attr_accessor :err
@@ -2045,8 +2067,6 @@ class GlarkOptions
     @file_names_only   = false      # display only the file names
     @filter            = true       # display only matches
     @invert_match      = false      # display non-matching lines
-    @basename          = nil        # match files with this basename
-    @fullname          = nil        # match files with this full name
     @nocase            = false      # match case
     @match_limit       = nil        # the maximum number of matches to display per file
     @local_config_files = false
@@ -2059,6 +2079,10 @@ class GlarkOptions
     @whole_lines       = false      # true means patterns must match the entire line
     @whole_words       = false      # true means all patterns are '\b'ed front and back
     @write_null        = false      # in @file_names_only mode, write '\0' instead of '\n'
+    @with_basename     = nil        # match files with this basename
+    @without_basename  = nil        # match files without this basename
+    @with_fullname     = nil        # match files with this fullname
+    @without_fullname  = nil        # match files without this fullname
     @show_break        = false      # whether to show the break between sections
     @split_as_path     = true       # whether to split arguments that include the path separator
 
@@ -2235,10 +2259,14 @@ class GlarkOptions
           @split_as_path = to_boolean(value)
         when "size-limit"
           @size_limit = value.to_i
-        when "basename", "name"
-          @basename = Regexp.create(value)
-        when "fullname", "path"
-          @fullname = Regexp.create(value)
+        when "basename", "name", "with-basename", "with-name"
+          @with_basename = Regexp.create(value)
+        when "fullname", "path", "with-fullname", "with-path"
+          @with_fullname = Regexp.create(value)
+        when "without-basename", "without-name"
+          @without_basename = Regexp.create(value)
+        when "without-fullname", "without-path"
+          @without_fullname = Regexp.create(value)
         end
       end
     end
@@ -2332,11 +2360,14 @@ class GlarkOptions
     when "--after"
       @range_start = args.shift
 
+    when %r{^--after(?:=(\d+%?))}
+      @range_start = $1
+      
       # before (range)
     when "-b", "--before"
       @range_end = args.shift
       
-    when %r{^--before(?:=(\d+%))}
+    when %r{^--before(?:=(\d+%?))}
       @range_end = $1
       
       # range
@@ -2430,16 +2461,23 @@ class GlarkOptions
       @file_names_only = true
       @invert_match = true
 
-      # For selecting by file base name, like find(1) --name
-    when %r{^--(?:basename|name)(?:=(.+))?$}
+      # basename include/exclude matching
+    when %r{^--(?:with-)?(?:basename|name)(?:=(.+))?$}
       arg = $1 || args.shift
-      # Make this a regexp.  If they want globs they can use find.
-      @basename = Regexp.create(arg.dup)
+      @with_basename = Regexp.create(arg.dup)
 
-      # For selecting by file full name, like find(1) --path
-    when %r{^--(?:fullname|path)(?:=(.+))?$}
+    when %r{^--without-(?:basename|name)(?:=(.+))?$}
       arg = $1 || args.shift
-      @fullname = Regexp.create(arg.dup)
+      @without_basename = Regexp.create(arg.dup)
+
+      # fullname include/exclude matching
+    when %r{^--(?:with-)?(?:fullname|path)(?:=(.+))?$}
+      arg = $1 || args.shift
+      @with_fullname = Regexp.create(arg.dup)
+
+    when %r{^--without-(?:fullname|path)(?:=(.+))?$}
+      arg = $1 || args.shift
+      @without_fullname = Regexp.create(arg.dup)
 
       # colors
     when "-T", %r{^--text-color(?:=(.+))?$}
@@ -2552,7 +2590,6 @@ class GlarkOptions
     when "--dump"
       fields = {
         "after" => @after,
-        "basename" => @basename,
         "before" => @before,
         "binary_files" => @binary_files,
         "count" => @count,
@@ -2564,7 +2601,6 @@ class GlarkOptions
         "file_highlight" => @file_highlight ? @file_highlight.highlight("filename") : "filename",
         "file_names_only" => @file_names_only,
         "filter" => @filter,
-        "fullname" => @fullname,
         "highlight" => @highlight,
         "invert_match" => @invert_match,
         "known_nontext_files" => FileTester.nontext_extensions.join(", "),
@@ -2587,6 +2623,10 @@ class GlarkOptions
         "version" => $VERSION,
         "whole_lines" => @whole_lines,
         "whole_words" => @whole_words,
+        "with-basename" => @with_basename,
+        "without-basename" => @without_basename,
+        "with-filename" => @with_filename,
+        "without-filename" => @without_filename,
         "write_null" => @write_null,
       }
 
